# Script: iteriolen.py
# Datum : 2024-01-21, Meindert Meindertsma

def itereer_over(bron):
    print("\n-----", bron.naam, "-----")

    try:  # is dit object iterabel?
        print("iter     :", iter(bron))
    except Exception as e:
        print("iter     :", e)

    try:  # is dit object een iterator?
        print("next     :", next(bron))
    except Exception as e:
        print("next     :", e)

    try:  # bestaat er enige iterator voor dit object?
        print("next/iter:", next(iter(bron)))
    except Exception as e:
        print("next/iter:", e)

    print("for      :", end=" ")
    try:  # verdere iteratie over dit object mogelijk?
        for pakketje in bron:
            print(pakketje, end=" ")
        print()
    except Exception as e:
        print(e)


#---- Niet iterabel, geen iterator ----#
class Type1_Bron:
    def __init__(self, naam, lijst):
      self.naam  = naam
      self.lijst = lijst

#---- Formeel iterabel, feitelijk niet, geen iterator ----#
class Type2_Bron(Type1_Bron):
    def __iter__(self):
        return self

#---- Niet iterabel, wel een iterator ----#
class Type3_Bron(Type1_Bron):
    def __next__(self):
        if len(self.lijst) > 0:
            return self.lijst.pop(0)
        raise StopIteration()

#---- Feitelijk iterabel, tevens een iterator ----#
class Type4_Bron(Type2_Bron, Type3_Bron):
    pass

#---- Formeel iterabel, feitelijk niet, wel een iterator ----#
class Type5_Bron(Type3_Bron):
    def __iter__(self):
        return self.lijst

#---- Feitelijk iterabel, maar
#     afgifte overgedragen aan externe iterator ----#
class Type6_Bron(Type1_Bron):
    def __iter__(self):
        return Type4_Bron("anoniem", [10*x for x in self.lijst])

#---- Feitelijk een iterabele iterator, maar
#     afgifte toch overgedragen aan externe iterator ----#
class Type7_Bron(Type3_Bron, Type6_Bron):
    pass

bron1 = Type1_Bron("bron1", [1, 2, 3, 4])
bron2 = Type2_Bron("bron2", [1, 2, 3, 4])
bron3 = Type3_Bron("bron3", [1, 2, 3, 4])
bron4 = Type4_Bron("bron4", [1, 2, 3, 4])
bron5 = Type5_Bron("bron5", [1, 2, 3, 4])
bron6 = Type6_Bron("bron6", [1, 2, 3, 4])
bron7 = Type7_Bron("bron7", [1, 2, 3, 4])

itereer_over(bron1)
itereer_over(bron2)
itereer_over(bron3)
itereer_over(bron4)
itereer_over(bron5)
itereer_over(bron6)
itereer_over(bron7)

